package com.jeeplus.test.cw.projectRecords.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.sys.constant.enums.LogTypeEnum;
import com.jeeplus.test.cw.projectRecords.service.CwProjectRecordsService;
import com.jeeplus.test.cw.projectRecords.service.dto.CwProjectRecordsDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * 财务项目登记controller
 * @author: 徐滕
 * @version: 2022-11-10 14:55
 */
@RestController
@Api("财务项目信息")
@RequestMapping(value = "/cwProjectRecords")
public class CwProjectRecordsController {

    @Resource
    private CwProjectRecordsService cwProjectRecordsService;

    /**
     * 查询财务项目信息列表
     * @param cwProjectRecordsDTO
     * @param page
     * @return
     */
    @ApiLog("查询财务项目信息列表")
    @PreAuthorize("hasAuthority('cwProjectRecords:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<CwProjectRecordsDTO>> data(CwProjectRecordsDTO cwProjectRecordsDTO, Page<CwProjectRecordsDTO> page) throws Exception {
        IPage<CwProjectRecordsDTO> result = new Page<CwProjectRecordsDTO>();
        result = cwProjectRecordsService.findList (page,cwProjectRecordsDTO);
        return ResponseEntity.ok (result);
    }

    /**
     * 查询财务项目详情
     * @param id
     * @return
     */
    @ApiLog("查询财务项目详情")
    @PreAuthorize ("hasAnyAuthority('cwProjectRecords:view','cwProjectRecords:add','cwProjectRecords:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        CwProjectRecordsDTO cwProjectRecordsDTO = cwProjectRecordsService.queryById ( id );
        return ResponseEntity.ok (cwProjectRecordsDTO);
    }

    /**
     * 保存财务项目信息
     * @param cwProjectRecordsDTO
     * @return
     */
    @ApiLog(value = "修改/新增财务项目信息", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectRecords:add','cwProjectRecords:edit')")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody CwProjectRecordsDTO cwProjectRecordsDTO) throws Exception {
        cwProjectRecordsService.save(cwProjectRecordsDTO);
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 保存财务项目信息-流程
     * @param cwProjectRecordsDTO
     * @return
     */
    @ApiLog(value = "保存财务项目信息-流程", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectRecords:add','cwProjectRecords:edit')")
    @PostMapping("saveForm")
    public ResponseEntity saveForm(@Valid @RequestBody CwProjectRecordsDTO cwProjectRecordsDTO) throws Exception {
        String id = cwProjectRecordsService.saveForm(cwProjectRecordsDTO);
        return ResponseUtil.newInstance().add("businessTable", "cw_project_records").add("businessId", id).ok ("操作成功");
    }

    /**
     * 删除财务项目信息
     * @param ids
     * @return
     */
    @ApiLog(value = "删除财务项目信息", type = LogTypeEnum.SAVE)
    @PreAuthorize ("hasAuthority('cwProjectRecords:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        return cwProjectRecordsService.deleteByIds(ids);
    }

    /**
     * 根据id修改状态值status
     * @param dto
     * @return
     */
    @ApiOperation(value = "根据id修改状态值status")
    @PostMapping(value = "/updateStatusById")
    public ResponseEntity<String> updateStatusById(@RequestBody CwProjectRecordsDTO dto) {
        String s = cwProjectRecordsService.updateStatusById(dto);
        return ResponseEntity.ok(s);
    }

}
