package com.jeeplus.test.cw.projectReportArchive.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.sys.constant.enums.LogTypeEnum;
import com.jeeplus.test.cw.projectReportArchive.service.CwProjectReportArchiveService;
import com.jeeplus.test.cw.projectReportArchive.service.dto.CwProjectReportArchiveDTO;
import com.jeeplus.test.cw.projectReportArchive.service.dto.ReportDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * 财务项目报告归档
 */
@RestController
@Api("财务项目报告归档")
@RequestMapping(value = "/cwProjectReportArchive")
public class CwProjectReportArchiveController {

    @Resource
    private CwProjectReportArchiveService cwProjectReportArchiveService;

    /**
     * 查询财务项目报告归档信息列表
     * @param cwProjectReportArchiveDTO
     * @param page
     * @return
     */
    @ApiLog("查询财务项目报告归档信息列表")
    @PreAuthorize("hasAuthority('cwProjectReportArchive:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<CwProjectReportArchiveDTO>> data(CwProjectReportArchiveDTO cwProjectReportArchiveDTO, Page<CwProjectReportArchiveDTO> page) throws Exception {
        IPage<CwProjectReportArchiveDTO> result = new Page<CwProjectReportArchiveDTO>();
        result = cwProjectReportArchiveService.findList (page, cwProjectReportArchiveDTO);
        return ResponseEntity.ok (result);
    }

    @GetMapping("reportList")
    public ResponseEntity<IPage<ReportDTO>> findReportList(ReportDTO reportDTO, Page<ReportDTO> page) throws Exception {
        IPage<ReportDTO> result = new Page<ReportDTO>();
        result = cwProjectReportArchiveService.findReportList (page, reportDTO);
        return ResponseEntity.ok (result);
    }

    /**
     * 查询财务项目报告归档详情
     * @param id
     * @return
     */
    @ApiLog("查询财务项目报告归档详情")
    @PreAuthorize ("hasAnyAuthority('cwProjectReportArchive:view','cwProjectReportArchive:add','cwProjectReportArchive:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        CwProjectReportArchiveDTO cwProjectReportArchiveDTO = cwProjectReportArchiveService.queryById ( id );
        return ResponseEntity.ok (cwProjectReportArchiveDTO);
    }

    /**
     * 保存财务项目报告归档信息
     * @param cwProjectReportArchiveDTO
     * @return
     */
    @ApiLog(value = "修改/新增财务项目报告归档信息", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectReportArchive:add','cwProjectReportArchive:edit')")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody CwProjectReportArchiveDTO cwProjectReportArchiveDTO) throws Exception {
        cwProjectReportArchiveService.save(cwProjectReportArchiveDTO);
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 保存财务项目报告归档信息-流程
     * @param cwProjectReportArchiveDTO
     * @return
     */
    @ApiLog(value = "保存财务项目报告归档信息-流程", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectReportArchive:add','cwProjectReportArchive:edit')")
    @PostMapping("saveForm")
    public ResponseEntity saveForm(@Valid @RequestBody CwProjectReportArchiveDTO cwProjectReportArchiveDTO) throws Exception {
        String id = cwProjectReportArchiveService.saveForm(cwProjectReportArchiveDTO);
        return ResponseUtil.newInstance().add("businessTable", "cw_project_report_archive").add("businessId", id).ok ("操作成功");
    }

    /**
     * 删除财务项目报告归档信息
     * @param ids
     * @return
     */
    @ApiLog(value = "删除财务项目报告归档信息", type = LogTypeEnum.SAVE)
    @PreAuthorize ("hasAuthority('cwProjectReportArchive:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        return cwProjectReportArchiveService.deleteByIds(ids);
    }

    /**
     * 根据id修改状态值status
     * @param dto
     * @return
     */
    @ApiOperation(value = "根据id修改状态值status")
    @PostMapping(value = "/updateStatusById")
    public ResponseEntity<String> updateStatusById(@RequestBody CwProjectReportArchiveDTO dto) {
        String s = cwProjectReportArchiveService.updateStatusById(dto);
        return ResponseEntity.ok(s);
    }

}
